package meerkat;

import meerkat.Registry.*;
import meerkat.bulletinboard.BulletinBoardClient;
import meerkat.crypto.DigitalSignature;
import meerkat.protobuf.BulletinBoardAPI.BulletinBoardMessage;
import meerkat.protobuf.BulletinBoardAPI.UnsignedBulletinBoardMessage;
import meerkat.protobuf.Crypto;

import java.io.InputStream;
import java.security.SignatureException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by Vladimir Eliezer Tokarev on 1/8/2016.
 * Gives the ability to synchronously manage voters information
 */
public class SimpleRegistry  implements VoterRegistry{

    protected DigitalSignature signer;
    protected BulletinBoardClient bulletinBoardClient ;
    protected Crypto.Signature  signature;
    protected InputStream certificateStream;

    public SimpleRegistry(DigitalSignature signer,
                          BulletinBoardClient communicator,
                          InputStream certificateStream) {
        this.signer = signer;
        this.bulletinBoardClient  = communicator;
        this.certificateStream = certificateStream;
    }

    /**
     * Creates BulletinBoardMessage with signed basicMessage and
     *         UnsignedBulletinBoardMessage that contains the basic message
     *
     * @param basicMessage BasicMessage
     * @return BulletinBoardMessage
     */
    private BulletinBoardMessage CreateBulletinBoardMessage(UnsignedBulletinBoardMessage basicMessage) {
        try {
            BulletinBoardMessage.Builder bulletinBoardMessage = BulletinBoardMessage.newBuilder();

            signer.updateContent(basicMessage);
            signature = signer.sign();
            bulletinBoardMessage.setMsg(basicMessage).addSig(signature);

            return bulletinBoardMessage.build();
        } catch (SignatureException e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
     * Tries to post basicMessage and return true if its been successfully posted
     *
     * @param Message the massage to post
     * @return true when the post was successful else false
     */
    private void SafePost(BulletinBoardMessage Message, BulletinBoardClient.ClientCallback callback) {
        bulletinBoardClient .postMessage(Message, callback);
    }

    public void AddVoter(RegistryMessages.VoterInfo voterInfo, RegistryCallBack callback) {
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder().
                        addTag(RegistryTags.ID_TAG + voterInfo.getId().getId())
                        .addTag(RegistryTags.VOTER_ENTRY_TAG.toString())
                        .addTag(RegistryTags.ACTION_TIMESTAMP_TAG + AccurateTimestamp.GetCurrentTimestampString());

        basicMessage.setData(voterInfo.getInfoBytes());

        SafePost(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void AddToGroup(RegistryMessages.VoterGroup voterGroup, RegistryCallBack callback) {
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterGroup.getVoterId().getId())
                        .addTag(RegistryTags.GROUP_ID_TAG + voterGroup.getGroupId().getId())
                        .addTag(RegistryTags.GROUP_ACTION_TAG .toString() + RegistryTags.ADD_TO_GROUP_TAG)
                        .addTag(RegistryTags.ACTION_TIMESTAMP_TAG + AccurateTimestamp.GetCurrentTimestampString());

        SafePost(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void SetVoted(RegistryMessages.VoterID voterId, RegistryCallBack callback) {
        UnsignedBulletinBoardMessage.Builder basicMessage =
                UnsignedBulletinBoardMessage.newBuilder()
                        .addTag(RegistryTags.ID_TAG + voterId.getId())
                        .addTag(RegistryTags.VOTE_ACTION_TAG.toString())
                        .addTag(RegistryTags.ACTION_TIMESTAMP_TAG + AccurateTimestamp.GetCurrentTimestampString());

        SafePost(CreateBulletinBoardMessage(basicMessage.build()), new BooleanCallBack(callback));
    }

    public void GetGroups(RegistryMessages.GroupID groupID, RegistryCallBack callback) {
        List<String> GroupsActionsTags = new ArrayList<String>() {{
           add(RegistryTags.GROUP_ID_TAG + groupID.getId());
        }};
        bulletinBoardClient.readMessages(CollectionMessagesUtils.GenerateFiltersFromTags(GroupsActionsTags),
                new RelevantDataCallBack(callback, signer, signature, certificateStream));
    }

    public void GetPersonIDDetails(RegistryMessages.VoterID voterID, RegistryCallBack callback) {
        List<String> GroupsActionsTags = new ArrayList<String>() {{
            add(RegistryTags.ID_TAG + voterID.getId());
            add(RegistryTags.VOTER_ENTRY_TAG.toString());
        }};
        bulletinBoardClient.readMessages(CollectionMessagesUtils.GenerateFiltersFromTags(GroupsActionsTags),
                new RelevantDataCallBack(callback, signer, signature, certificateStream));
    }
}
